#!/usr/bin/env python

#########################################################################
# This program is free software: you can redistribute it and/or modify  #
# it under the terms of the version 3 of the GNU General Public License #
# as published by the Free Software Foundation.                         #
#                                                                       #
# This program is distributed in the hope that it will be useful, but   #
# WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      #
# General Public License for more details.                              #
#                                                                       #
# You should have received a copy of the GNU General Public License     #
# along with this program. If not, see <http://www.gnu.org/licenses/>.  #
#                                                                       #
# Written by and Copyright (C) Francois Fleuret                         #
# Contact <francois.fleuret@idiap.ch> for comments & bug reports        #
#########################################################################

import subprocess

from torch import nn
from torch.nn import functional as fn
from torch import Tensor
from torch.nn import Module

import agtree2dot

class MLP(Module):
    def __init__(self, input_dim, hidden_dim, output_dim):
        super(MLP, self).__init__()
        self.fc1 = nn.Linear(input_dim, hidden_dim)
        self.fc2 = nn.Linear(hidden_dim, output_dim)

    def forward(self, x):
        x = self.fc1(x)
        x = fn.tanh(x)
        x = self.fc2(x)
        return x

mlp = MLP(10, 20, 1)
input = Tensor(100, 10).normal_()
target = Tensor(100, 1).normal_()
output = mlp(input)
criterion = nn.MSELoss()
loss = criterion(output, target)

agtree2dot.save_dot(loss,
                    {
                        input: 'input',
                        target: 'target',
                        loss: 'loss',
                        mlp.fc1.weight: 'weight1',
                        mlp.fc1.bias: 'bias1',
                        mlp.fc2.weight: 'weight2',
                        mlp.fc2.bias: 'bias2',
                    },
                    open('./mlp.dot', 'w'))

print('Generated mlp.dot')

try:

    fontname = 'Computer Modern'
    fontsize = 12
    subprocess.check_call(['dot', 'mlp.dot',
                           '-Lg',
                           '-T', 'pdf',
                           '-Efontname=' + fontname, '-Efontsize=' + str(fontsize),
                           '-Nfontname=' + fontname, '-Nfontsize=' + str(fontsize),
                           '-o', 'mlp.pdf' ])

except subprocess.CalledProcessError:

    print('Calling the dot command failed. Is Graphviz installed?')
    sys.exit(1)

print('Generated mlp.pdf')
